@@ -8,14 +8,14 @@ import ssl
 import sys
 import time
 import zlib
+from _socket import SHUT_RDWR
 from enum import Enum
 from pathlib import Path
-from _socket import SHUT_RDWR
 
 import pem
-from devo.common import Configuration, get_log, get_stream_handler
 from OpenSSL import SSL, crypto
 
+from devo.common import Configuration, get_log, get_stream_handler
 from .transformsyslog import (COMPOSE, COMPOSE_BYTES, FACILITY_USER, FORMAT_MY,
                               FORMAT_MY_BYTES, SEVERITY_INFO, priority_map)
 
@@ -23,38 +23,57 @@ PYPY = hasattr(sys, 'pypy_version_info')
 
 
 class ERROR_MSGS(str, Enum):
-    WRONG_FILE_TYPE = "'%s' is not a valid type to be opened as a file"
-    ADDRESS_TUPLE = "Devo-SenderConfigSSL| address must be a tuple (\"hostname\", int(port))'"
-    WRONG_SSL_CONFIG = "Devo-SenderConfigSSL|Can't create SSL config: %s"
-    CONFIG_FILE_NOT_FOUND = "Error in the configuration, %s is not a file or the path does not exist"
-    CANT_READ_CONFIG_FILE = "Error in the configuration %s can't be read\noriginal error: %s"
-    CONFIG_FILE_PROBLEM = "Error in the configuration, %s problem related to: %s"
-    KEY_NOT_COMPATIBLE_WITH_CERT = "Error in the configuration, the key: %s is not compatible with the cert: %s\noriginal error: %s"
-    CHAIN_NOT_COMPATIBLE_WITH_CERT = "Error in config, the chain: %s is not compatible with the certificate: %s\noriginal error: %s"
-    TIMEOUT_RELATED_TO_AN_INCORRECT_ADDRESS_PORT = "Possible error in config, a timeout could be related to an incorrect address/port: %s\noriginal error: %s"
-    INCORRECT_ADDRESS_PORT = "Error in config, incorrect address/port: %s\noriginal error: %s"
-    CERTIFICATE_IN_ADDRESS_IS_NOT_COMPATIBLE = "Error in config, the certificate in the address: %s is not compatible with: %s"
-    ADDRESS_MUST_BE_A_TUPLE = "Devo-SenderConfigSSL| address must be a tuple '(\"hostname\", int(port))'"
-    CANT_CREATE_TCP_CONFIG = "DevoSenderConfigTCP|Can't create TCP config: %s"
-    PROBLEMS_WITH_SENDER_ARGS = "Problems with args passed to Sender"
-    TCP_CONN_ESTABLISHMENT_SOCKET = "TCP conn establishment socket error: %s"
-    SSL_CONN_ESTABLISHMENT_SOCKET = "SSL conn establishment socket error: %s"
-    PFX_CERTIFICATE_READ_FAILED = "PFX Certificate read failed: %s"
-    SEND_ERROR = "Send error"
-    SOCKET_ERROR = "Socket error: %s"
-    SOCKET_CANT_CONNECT_UNKNOWN_ERROR = "Socket cant connect: unknown error"
-    NO_ADDRESS = "No address"
+    def __str__(self):
+        return str(self.value)
+
+    WRONG_FILE_TYPE = "'%s' is not a valid type to be opened as a file",
+    ADDRESS_TUPLE = "Devo-SenderConfigSSL| address must be a tuple (\"hostname\", int(port))'",
+    WRONG_SSL_CONFIG = "Devo-SenderConfigSSL|Can't create SSL config: %s",
+    CONFIG_FILE_NOT_FOUND = "Error in the configuration, %s is not a file or the path does not" \
+                            " exist",
+    CANT_READ_CONFIG_FILE = "Error in the configuration %s can't be read\noriginal error: %s",
+    CONFIG_FILE_PROBLEM = "Error in the configuration, %s problem related to: %s",
+    KEY_NOT_COMPATIBLE_WITH_CERT = "Error in the configuration, the key: %s is not compatible" \
+                                   " with the cert: %s\noriginal error: %s",
+    CHAIN_NOT_COMPATIBLE_WITH_CERT = "Error in config, the chain: %s is not compatible with the" \
+                                     " certificate: %s\noriginal error: %s",
+    TIMEOUT_RELATED_TO_AN_INCORRECT_ADDRESS_PORT = "Possible error in config, a timeout could be" \
+                                                   " related to an incorrect address/port: %s\n" \
+                                                   "original error: %s",
+    INCORRECT_ADDRESS_PORT = "Error in config, incorrect address/port: %s\noriginal error: %s",
+    CERTIFICATE_IN_ADDRESS_IS_NOT_COMPATIBLE = "Error in config, the certificate in the address:" \
+                                               " %s is not compatible with: %s",
+    ADDRESS_MUST_BE_A_TUPLE = "Devo-SenderConfigSSL| address must be a tuple '(\"hostname\"," \
+                              " int(port))'",
+    CANT_CREATE_TCP_CONFIG = "DevoSenderConfigTCP|Can't create TCP config: %s",
+    PROBLEMS_WITH_SENDER_ARGS = "Problems with args passed to Sender",
+    TCP_CONN_ESTABLISHMENT_SOCKET = "TCP conn establishment socket error: %s",
+    SSL_CONN_ESTABLISHMENT_SOCKET = "SSL conn establishment socket error: %s",
+    PFX_CERTIFICATE_READ_FAILED = "PFX Certificate read failed: %s",
+    SEND_ERROR = "Send error",
+    SOCKET_ERROR = "Socket error: %s",
+    SOCKET_CANT_CONNECT_UNKNOWN_ERROR = "Socket cant connect: unknown error",
+    NO_ADDRESS = "No address",
+    MULTILINE_SENDING_ERROR = "Error sending multiline event: %s",
+    RAW_SENDING_ERROR = "Error sending raw event: %s",
+    CLOSING_ERROR = "Error closing connection"
+    FLUSHING_BUFFER_ERROR = "Error flushing buffer"
 
 
 class DevoSenderException(Exception):
-    """ Default Devo Sender Exception """
+    """ Default Devo Sender Exception for functionalities related to sending
+     events to the platform"""
 
-    def __init__(self, message):
-        self.message = message
-        super().__init__(message)
+    def __init__(self, message: str):
+        """
+        Creates an exception related to event sending tasks
 
-    def __str__(self):
-        return self.message
+        :param message: Message describing the exception. It will be also
+         used as `args` attribute in `Exception` class
+        """
+        self.message: str = message
+        """Message describing exception"""
+        super().__init__(self.message)
 
 
 class SenderConfigSSL:
@@ -121,16 +140,16 @@ class SenderConfigSSL:
                         file).is_file()):
                     raise DevoSenderException(
                         ERROR_MSGS.CONFIG_FILE_NOT_FOUND % file)
-            except IOError as message:
-                if message.errno == errno.EACCES:
+            except IOError as error:
+                if error.errno == errno.EACCES:
                     raise DevoSenderException(
                         ERROR_MSGS.CANT_READ_CONFIG_FILE % (
-                            file, str(message))) \
-                        from message
+                            file, str(error))) \
+                        from error
                 else:
                     raise DevoSenderException(
-                        ERROR_MSGS.CONFIG_FILE_PROBLEM % (file, str(message))) \
-                        from message
+                        ERROR_MSGS.CONFIG_FILE_PROBLEM % (file, str(error))) \
+                        from error
         return True
 
     def check_config_certificate_key(self):
@@ -155,11 +174,11 @@ class SenderConfigSSL:
             context.use_certificate(certificate_obj)
         try:
             context.check_privatekey()
-        except SSL.Error as message:
+        except SSL.Error as error:
             raise DevoSenderException(
                 ERROR_MSGS.KEY_NOT_COMPATIBLE_WITH_CERT % (
-                    self.key, self.cert, str(message)
-                )) from message
+                    self.key, self.cert, str(error)
+                )) from error
         return True
 
     def check_config_certificate_chain(self):
@@ -185,11 +204,11 @@ class SenderConfigSSL:
                 certificates_chain, certificate_obj)
         try:
             store_ctx.verify_certificate()
-        except crypto.X509StoreContextError as message:
+        except crypto.X509StoreContextError as error:
             raise DevoSenderException(
                 ERROR_MSGS.CHAIN_NOT_COMPATIBLE_WITH_CERT % (
-                    self.chain, self.cert, str(message)
-                )) from message
+                    self.chain, self.cert, str(error)
+                )) from error
         return True
 
     def check_config_certificate_address(self):
@@ -206,16 +225,16 @@ class SenderConfigSSL:
         connection = SSL.Connection(context, sock)
         try:
             connection.connect(self.address)
-        except socket.timeout as message:
+        except socket.timeout as error:
             raise DevoSenderException(
                 ERROR_MSGS.TIMEOUT_RELATED_TO_AN_INCORRECT_ADDRESS_PORT % (
-                    str(self.address), str(message)
-                )) from message
-        except ConnectionRefusedError as message:
+                    str(self.address), str(error)
+                )) from error
+        except ConnectionRefusedError as error:
             raise DevoSenderException(
                 ERROR_MSGS.INCORRECT_ADDRESS_PORT % (
-                    str(self.address), str(message)
-                )) from message
+                    str(self.address), str(error)
+                )) from error
         sock.setblocking(True)
         connection.do_handshake()
         server_chain = connection.get_peer_cert_chain()
@@ -303,7 +322,8 @@ class Sender(logging.Handler):
     :param con_type: TCP or SSL, default SSL, you can pass it in
     config object too
     :param timeout: timeout for socket
-    :param inactivity_timeout: inactivity timeout for Ingestion balancer, so connection is restarted before reaching
+    :param inactivity_timeout: inactivity timeout for Ingestion balancer, so connection is
+     restarted before reaching
     :param debug: For more info in console/logger output
     :param logger: logger. Default sys.console
     """
@@ -422,6 +442,7 @@ class Sender(logging.Handler):
                     )
             else:
                 self.socket = ssl.wrap_socket(self.socket,
+                                              ssl_version=ssl.PROTOCOL_TLS,
                                               cert_reqs=ssl.CERT_NONE)
 
             self.socket.connect(self._sender_config.address)
@@ -530,8 +551,9 @@ class Sender(logging.Handler):
             self.close()
             return False
 
-        # If there is no activity (connection or message sent) for an amount of time bigger then the inactivity
-        # timeout, the balancer may have already close the connection. Close it and reconnect.
+        # If there is no activity (connection or message sent) for an amount of time bigger
+        # then the inactivity timeout, the balancer may have already close the connection.
+        # Close it and reconnect.
         if int(time.time()) - self.last_message > self.inactivity_timeout:
             self.close()
             return False
@@ -545,8 +567,8 @@ class Sender(logging.Handler):
         if self.socket is not None:
             try:
                 self.socket.shutdown(SHUT_RDWR)
-            except Exception as e:  # Try else continue
-                pass
+            except Exception:  # Try else continue
+                logging.exception(ERROR_MSGS.CLOSING_ERROR)
             self.socket.close()
             self.socket = None
 
@@ -556,7 +578,7 @@ class Sender(logging.Handler):
             record = Sender.__encode_record(record)
             return b'%d %s' % (len(record), record)
         except Exception as error:
-            raise DevoSenderException(error) from error
+            raise DevoSenderException(ERROR_MSGS.MULTILINE_SENDING_ERROR % str(error)) from error
 
     @staticmethod
     def __encode_record(record):
@@ -616,15 +638,17 @@ class Sender(logging.Handler):
                     return 0
                 except socket.error as error:
                     self.close()
-                    raise DevoSenderException(
-                        ERROR_MSGS.SOCKET_ERROR % str(error)) from error
+                    raise DevoSenderException(ERROR_MSGS.SOCKET_ERROR % str(error)) from error
                 finally:
                     if self.debug:
-                        self.logger.debug('sent|%d|size|%d|msg|%s' %
-                                          (sent, len(record), record))
-            raise Exception(ERROR_MSGS.SOCKET_CANT_CONNECT_UNKNOWN_ERROR)
+                        self.logger.debug(
+                            'sent|%d|size|%d|msg|%s' % (sent, len(record), record)
+                        )
+            raise DevoSenderException(ERROR_MSGS.SOCKET_CANT_CONNECT_UNKNOWN_ERROR)
+        except DevoSenderException:
+            raise
         except Exception as error:
-            raise DevoSenderException(error) from error
+            raise DevoSenderException(ERROR_MSGS.RAW_SENDING_ERROR % str(error)) from error
 
     @staticmethod
     def compose_mem(tag, **kwargs):
@@ -732,13 +756,12 @@ class Sender(logging.Handler):
             try:
                 compressor = zlib.compressobj(self.buffer.compression_level,
                                               zlib.DEFLATED, 31)
-                record = compressor.compress(self.buffer.text_buffer) \
-                         + compressor.flush()
+                record = compressor.compress(self.buffer.text_buffer) + compressor.flush()
                 if self.send_raw(record, zip=True):
                     return self.buffer.events
                 return 0
             except Exception as error:
-                raise error
+                raise DevoSenderException(ERROR_MSGS.FLUSHING_BUFFER_ERROR) from error
             finally:
                 self.buffer.text_buffer = b''
                 self.buffer.events = 0
