@@ -5,6 +5,8 @@ import re
 import sys
 import time
 
+from devo.sender.data import open_file
+
 
 def find_key_index(value=None, headers=None):
     """Find index of key value in a list"""
@@ -81,23 +83,18 @@ class Lookup:
                                            type_of_key=type_of_key)
         self.send_control(event=event, headers=p_headers, action=action)
 
-    def send_data_line(self, key=None, fields=None,
+    def send_data_line(self, fields=None,
                        delete=False, key_index=None):
         """
         Send only the data
-        :param key: key value, optional if you send 'key_index'
-        :param key_index: index of key in headers. Optional if you send 'key'
+        :param key_index: index of key in headers
         :param fields: list with fields values
         :param delete: action for field: 'delete' to delete field with this key
                        or add to add value
         :return:
         """
-        # TODO: Deprecate this if with list_to_fields in v4
-        p_fields = Lookup.list_to_fields(fields=fields, key=key,
-                                         escape_quotes=self.escape_quotes) \
-            if key_index is None \
-            else Lookup.process_fields(fields=fields, key_index=key_index,
-                                       escape_quotes=self.escape_quotes)
+        p_fields = Lookup.process_fields(fields=fields, key_index=key_index,
+                                         escape_quotes=self.escape_quotes)
         self.send_data(row=p_fields, delete=delete)
 
     @staticmethod
@@ -151,7 +148,7 @@ class Lookup:
         if self.escape_quotes:
             return False
 
-        with open(file_path) as f:
+        with open_file(file_path) as f:
             line = next((l_temp for l_temp in f if '"' in l_temp), None)
 
         return True if line is not None else False
@@ -163,7 +160,7 @@ class Lookup:
                  types=None, detect_types=False):
         """Send CSV file to lookup
 
-        :param path: The path to CSV file
+        :param path: The path to CSV file (can be `str` or `Path`)
         :param has_header: If the file has header to avoid it (default True)
         :param delimiter: CSV delimiter (default ;)
         :param quotechar: CSV quotechar (default ")
@@ -177,7 +174,7 @@ class Lookup:
         """
         try:
             if detect_types:
-                with open(path, 'r') as csv_file:
+                with open_file(path, mode='r') as csv_file:
                     spam_reader = csv.reader(csv_file, delimiter=delimiter,
                                              quotechar=quotechar)
 
@@ -189,7 +186,7 @@ class Lookup:
                             sys.exc_info()[0])
 
         try:
-            with open(path, 'r') as csv_file:
+            with open_file(path, mode='r') as csv_file:
                 spam_reader = csv.reader(csv_file, delimiter=delimiter,
                                          quotechar=quotechar)
                 # Conform headers list
@@ -229,7 +226,7 @@ class Lookup:
                             escape_quotes=self.escape_quotes)
                         self.send_data(row=p_fields,
                                        delete=field_action == "delete"
-                                       or field_action == "DELETE")
+                                              or field_action == "DELETE")
 
                         # Send full log for historic
                         if historic_tag is not None:
@@ -353,8 +350,8 @@ class Lookup:
                                                         else type_of_key)
         elif key is not None:
             out = '[{"%s":{"type":"%s","key":true}}' % (key, types[key_index]
-                                                        if key_index and types
-                                                        else type_of_key)
+            if key_index and types
+            else type_of_key)
         else:
             raise Exception("Not key identified")
 
@@ -397,31 +394,6 @@ class Lookup:
             out += Lookup.field_to_str(item, escape_quotes)
         return out
 
-    # TODO: Deprecated
-    @staticmethod
-    def list_to_fields(fields=None, key="key", escape_quotes=False):
-        """
-        Transform list item to the object we need send to Devo for each row
-        :param list fields: list of field names
-        :param str key: key name, optional
-        :result str
-        """
-        key = Lookup.clean_field(key, escape_quotes)
-        # First the key
-        out = '%s' % key
-
-        if fields is None:
-            return out
-
-        # The rest of the fields
-        for item in fields:
-            item = Lookup.clean_field(item, escape_quotes)
-            # If file is the key don't add
-            if item == key:
-                continue
-            out += ',%s' % item
-        return out
-
     @staticmethod
     def clean_field(field=None, escape_quotes=False):
         """
